from typing import Any, Optional

import httpx


class TaigaAPI:
    def __init__(self, base_url: str, token: Optional[str] = None):
        self.base_url = base_url.rstrip("/")
        self.token = token
        self.client = httpx.Client(timeout=30.0)

    def set_token(self, token: str):
        self.token = token

    def _headers(self) -> dict:
        headers = {"Content-Type": "application/json"}
        if self.token:
            headers["Authorization"] = f"Bearer {self.token}"
        return headers

    def get(self, endpoint: str, params: dict = None) -> Any:
        resp = self.client.get(
            f"{self.base_url}/{endpoint}", headers=self._headers(), params=params
        )
        resp.raise_for_status()
        return resp.json()

    def post(self, endpoint: str, data: dict) -> Any:
        resp = self.client.post(f"{self.base_url}/{endpoint}", headers=self._headers(), json=data)
        resp.raise_for_status()
        return resp.json()

    def patch(self, endpoint: str, data: dict) -> Any:
        resp = self.client.patch(f"{self.base_url}/{endpoint}", headers=self._headers(), json=data)
        resp.raise_for_status()
        return resp.json()

    def delete(self, endpoint: str) -> int:
        resp = self.client.delete(f"{self.base_url}/{endpoint}", headers=self._headers())
        resp.raise_for_status()
        return resp.status_code



import re

import httpx
import typer

from .api import TaigaAPI
from .config import (
    get_taiga_api_url,
    get_taiga_project_id,
    load_refresh_token,
    load_token,
    save_config_vars,
    save_refresh_token,
    save_token,
)
from .enums import (
    get_epic_status_id_by_name,
    get_task_status_id_by_name,
    get_user_story_status_id_by_name,
)
from .services import TaigaService

app = typer.Typer()


# --- CLI Commands ---


@app.command()
def configure():
    """
    Configura o Taiga CLI solicitando TAIGA_API_URL e
    TAIGA_PROJECT_ID e salvando em arquivo de configuração.
    """
    api_url = typer.prompt("Informe a URL da API do Taiga (ex: http://localhost:8000/api/v1/)")
    project_id = typer.prompt("Informe o slug ou ID do projeto Taiga")
    save_config_vars(api_url, project_id)
    typer.echo("[OK] Configuração salva em ~/.config/taiga-cli/config.env")


@app.command()
def list_projects():
    """Lista todos os projetos acessíveis ao usuário autenticado."""
    api_url = get_taiga_api_url()
    token = load_token()
    if not api_url:
        typer.echo("TAIGA_API_URL não definido no .env")
        raise typer.Exit(1)
    try:
        url = f"{api_url.rstrip('/')}/projects"
        headers = {"Authorization": f"Bearer {token}"} if token else {}
        resp = httpx.get(url, headers=headers, timeout=30.0)
        resp.raise_for_status()
        data = resp.json()
        if not data:
            typer.echo("Nenhum projeto encontrado.")
            return
        for proj in data:
            typer.echo(f"ID: {proj['id']} | Nome: {proj['name']} | Slug: {proj['slug']}")
    except Exception as e:
        typer.echo(f"[ERRO] Falha ao listar projetos: {e}")
        raise typer.Exit(1)


@app.command()
def login():
    """Solicita usuário e senha, autentica no Taiga e salva o token de acesso e refresh_token."""
    api_url = get_taiga_api_url()
    if not api_url:
        typer.echo("TAIGA_API_URL não definido. Execute 'taiga configure' primeiro.")
        raise typer.Exit(1)

    username = typer.prompt("Usuário do Taiga")
    password = typer.prompt("Senha do Taiga", hide_input=True)

    try:
        resp = httpx.post(
            f"{api_url.rstrip('/')}/auth",
            json={"type": "normal", "username": username, "password": password},
            timeout=30.0,
        )
        resp.raise_for_status()
        data = resp.json()
        token = data.get("auth_token")
        refresh_token = data.get("refresh") or data.get("refresh_token")
        save_token(token)
        typer.echo("[OK] Token salvo em ~/.config/taiga-cli/token")
        if refresh_token:
            save_refresh_token(refresh_token)
            typer.echo("[OK] Refresh token salvo em ~/.config/taiga-cli/refresh_token")
    except Exception as e:
        typer.echo(f"[ERRO] Falha ao autenticar: {e}")
        raise typer.Exit(1)


def ensure_token():
    """Garante que o token está válido, usando refresh_token se necessário."""
    token = load_token()
    if token:
        return token
    refresh_token = load_refresh_token()
    if not refresh_token:
        typer.echo("Token expirado ou ausente. Faça login novamente.")
        raise typer.Exit(1)
    api_url = get_taiga_api_url()
    try:
        resp = httpx.post(
            f"{api_url.rstrip('/')}/auth/refresh",
            json={"refresh": refresh_token},
            timeout=30.0,
        )
        resp.raise_for_status()
        data = resp.json()
        token = data.get("auth_token") or data.get("access")
        new_refresh = data.get("refresh") or data.get("refresh_token")
        if token:
            save_token(token)
        if new_refresh:
            save_refresh_token(new_refresh)
        typer.echo("[OK] Token renovado com sucesso.")
        return token
    except Exception as e:
        typer.echo(f"[ERRO] Falha ao renovar token: {e}")
        raise typer.Exit(1)


@app.command()
def get_project_id():
    """Busca o ID numérico do projeto Taiga pelo slug informado em TAIGA_PROJECT_ID."""
    project_id = get_taiga_project_id()
    api_url = get_taiga_api_url()
    token = load_token()
    if not project_id:
        typer.echo("TAIGA_PROJECT_ID não definido no .env")
        raise typer.Exit(1)
    try:
        url = f"{api_url.rstrip('/')}/projects/by_slug?slug={project_id}"
        headers = {"Authorization": f"Bearer {token}"} if token else {}
        resp = httpx.get(url, headers=headers, timeout=30.0)
        resp.raise_for_status()
        data = resp.json()
        typer.echo(f"ID numérico do projeto '{project_id}': {data['id']}")
    except Exception as e:
        typer.echo(f"[ERRO] Falha ao buscar ID do projeto: {e}")
        raise typer.Exit(1)


@app.command()
def hello():
    """Test CLI."""
    typer.echo("Taiga CLI is working!")


@app.command()
def list_epics():
    """List all epics in the project."""
    token = load_token()
    api_url = get_taiga_api_url()
    project_id = get_taiga_project_id()
    # ...
    api = TaigaAPI(api_url, token)
    service = TaigaService(api)
    if not project_id:
        typer.echo("TAIGA_PROJECT_ID not set in .env")
        raise typer.Exit(1)
    epics = service.list_epics(int(project_id))
    for e in epics:
        typer.echo(f"[{e.id}] {e.subject} | Status: {e.status}")


@app.command()
def list_initiatives():
    """List all initiatives (swimlanes) in the project."""
    api_url = get_taiga_api_url()
    token = load_token()
    project_id = get_taiga_project_id()
    api = TaigaAPI(api_url, token)
    service = TaigaService(api)
    if not project_id:
        typer.echo("TAIGA_PROJECT_ID not set in .env")
        raise typer.Exit(1)
    initiatives = service.list_initiatives(int(project_id))
    for i in initiatives:
        typer.echo(f"[{i.id}] {i.name} | Order: {i.order}")


@app.command()
def update_task(task_id: str, status: str = typer.Option(None), subject: str = typer.Option(None)):
    """Update a task by ID."""
    token = ensure_token()
    api_url = get_taiga_api_url()
    # ...
    api = TaigaAPI(api_url, token)
    data = {}
    if status:
        try:
            data["status"] = get_task_status_id_by_name(status)
        except ValueError as e:
            typer.echo(f"[ERRO] {e}")
            raise typer.Exit(1)
    if subject:
        data["subject"] = subject
    try:
        api.patch(f"tasks/{task_id}", data)
        typer.echo(f"[OK] Task {task_id} updated.")
    except httpx.HTTPStatusError as e:
        typer.echo(f"[ERRO] Falha ao atualizar task: {e.response.text}")
        raise typer.Exit(1)


@app.command()
def update_story(
    story_id: str, status: str = typer.Option(None), subject: str = typer.Option(None)
):
    """Update a story by ID."""
    token = ensure_token()
    api_url = get_taiga_api_url()
    project_id = get_taiga_project_id()
    # ...
    if not project_id:
        typer.echo("TAIGA_PROJECT_ID não definido.")
        raise typer.Exit(1)

    import re

    # Permitir busca por tag TS-XXX
    if re.fullmatch(r"TS-\d+", story_id, re.IGNORECASE):
        api = TaigaAPI(api_url, token)
        service = TaigaService(api)
        try:
            story = service.get_story_by_tag_full(story_id, int(project_id))
        except ValueError as e:
            typer.echo(f"[ERRO] {e}")
            raise typer.Exit(1)
        real_id = story["id"]
        version = story["version"]
    else:
        real_id = story_id
        version = None

    api = TaigaAPI(api_url, token)
    data = {}
    if version is not None:
        data["version"] = version
    if status:
        try:
            data["status"] = get_user_story_status_id_by_name(status)
        except ValueError as e:
            typer.echo(f"[ERRO] {e}")
            raise typer.Exit(1)
    if subject:
        data["subject"] = subject

    try:
        resp = api.client.patch(
            f"{api_url.rstrip('/')}/userstories/{real_id}", headers=api._headers(), json=data
        )
        resp.raise_for_status()
        typer.echo(f"[OK] Story {real_id} updated.")
    except httpx.HTTPStatusError as e:
        typer.echo(f"[ERRO] Falha ao atualizar story: {e.response.text}")
        raise typer.Exit(1)


@app.command()
def update_epic(
    epic_tag: str,
    status: str = typer.Option(None, help="Novo status (nome, ex: 'In progress')"),
    status_id: int = typer.Option(None, help="Novo status (id numérico)"),
    subject: str = typer.Option(None, help="Novo título do épico"),
    description: str = typer.Option(None, help="Nova descrição do épico"),
    assigned_to: int = typer.Option(None, help="ID do usuário responsável"),
    tags: str = typer.Option(None, help="Tags separadas por vírgula"),
):
    """
    Atualiza um épico pela tag (ex: EP-02).
    Permite atualizar status, título, descrição, responsável e tags.
    """
    project_id = get_taiga_project_id()
    if not project_id:
        typer.echo("TAIGA_PROJECT_ID not set in .env")
        raise typer.Exit(1)

    if not re.fullmatch(r"EP-\d+", epic_tag, re.IGNORECASE):
        typer.echo("[ERRO] O código do épico deve estar no formato 'EP-XX', onde XX são dígitos.")
        raise typer.Exit(1)

    if not (status or status_id or subject or description or assigned_to or tags):
        typer.echo(
            "[ERRO] Informe pelo menos um campo para atualizar: --status, --status-id,",
            "--subject, --description, --assigned-to, --tags.",
        )
        raise typer.Exit(1)

    token = ensure_token()
    api_url = get_taiga_api_url()
    api = TaigaAPI(api_url, token)
    service = TaigaService(api)
    try:
        epic = service.get_epic_by_tag_full(epic_tag, int(project_id))
    except ValueError as e:
        typer.echo(f"[ERRO] {e}")
        raise typer.Exit(1)

    payload = {"version": epic["version"]}

    # Resolução do status
    if status_id is not None:
        payload["status"] = status_id
    elif status:
        try:
            payload["status"] = get_epic_status_id_by_name(status)
        except ValueError as e:
            typer.echo(f"[ERRO] {e}")
            raise typer.Exit(1)

    if subject:
        payload["subject"] = subject
    if description:
        payload["description"] = description
    if assigned_to is not None:
        payload["assigned_to"] = assigned_to
    if tags:
        tags_list = [t.strip() for t in tags.split(",") if t.strip()]
        payload["tags"] = tags_list

    # ...
    if not token:
        typer.echo("[ERRO] Token não encontrado. Faça login novamente com 'taiga login'.")
        raise typer.Exit(1)
    try:
        api.patch(f"epics/{epic['id']}", payload)
        typer.echo(f"[OK] Épico {epic_tag} (id={epic['id']}) atualizado com sucesso.")
    except httpx.HTTPStatusError as e:
        typer.echo(f"[ERRO] Falha ao atualizar épico: {e.response.text}")
        raise typer.Exit(1)


@app.command()
def update_initiative(
    initiative_id: str, name: str = typer.Option(None), order: int = typer.Option(None)
):
    """Update an initiative (swimlane) by ID."""
    token = ensure_token()
    api_url = get_taiga_api_url()
    # ...
    if not token:
        typer.echo("[ERRO] Token não encontrado. Faça login novamente com 'taiga login'.")
        raise typer.Exit(1)
    api = TaigaAPI(api_url, token)
    data = {}
    if name:
        data["name"] = name
    if order is not None:
        data["order"] = order
    try:
        api.patch(f"swimlanes/{initiative_id}", data)
        typer.echo(f"[OK] Initiative {initiative_id} updated.")
    except httpx.HTTPStatusError as e:
        typer.echo(f"[ERRO] Falha ao atualizar initiative: {e.response.text}")
        raise typer.Exit(1)
    typer.echo(f"Initiative {initiative_id} updated.")



from enum import Enum


class UserStoryStatus(Enum):
    BACKLOG = 14
    READY = 15
    IN_PROGRESS = 16
    REVIEW = 17
    DONE = 18
    ARCHIVED = 19


_USER_STORY_STATUS_NAME_MAP = {
    "backlog": UserStoryStatus.BACKLOG,
    "ready": UserStoryStatus.READY,
    "in progress": UserStoryStatus.IN_PROGRESS,
    "review": UserStoryStatus.REVIEW,
    "done": UserStoryStatus.DONE,
    "archived": UserStoryStatus.ARCHIVED,
}


def get_user_story_status_id_by_name(name: str) -> int:
    key = name.strip().lower().replace("_", " ")
    status = _USER_STORY_STATUS_NAME_MAP.get(key)
    if status:
        return status.value
    raise ValueError(
        f"Status '{name}' não encontrado. Válidos: {', '.join(_USER_STORY_STATUS_NAME_MAP.keys())}"
    )


# Enum estático de status de task para o projeto Hermes Conrad
# Gerado a partir da API /api/v1/task-statuses?project=3
class TaskStatus(Enum):
    NEW = 11
    IN_PROGRESS = 12
    READY_FOR_TEST = 13
    CLOSED = 14
    NEEDS_INFO = 15


_TASK_STATUS_NAME_MAP = {
    "new": TaskStatus.NEW,
    "in progress": TaskStatus.IN_PROGRESS,
    "ready for test": TaskStatus.READY_FOR_TEST,
    "closed": TaskStatus.CLOSED,
    "needs info": TaskStatus.NEEDS_INFO,
}


def get_task_status_id_by_name(name: str) -> int:
    key = name.strip().lower().replace("_", " ")
    status = _TASK_STATUS_NAME_MAP.get(key)
    if status:
        return status.value
    raise ValueError(
        f"Status '{name}' não encontrado. Válidos: {', '.join(_TASK_STATUS_NAME_MAP.keys())}"
    )


# Enum estático de status de épico para o projeto Hermes Conrad
# Gerado a partir da API /api/v1/epic-statuses?project=3


class EpicStatus(Enum):
    NEW = 11
    IN_PROGRESS = 13
    READY = 12
    DONE = 15
    READY_FOR_TEST = 14


_EPIC_STATUS_NAME_MAP = {
    "new": EpicStatus.NEW,
    "in progress": EpicStatus.IN_PROGRESS,
    "ready": EpicStatus.READY,
    "done": EpicStatus.DONE,
    "ready for test": EpicStatus.READY_FOR_TEST,
}


def get_epic_status_id_by_name(name: str) -> int:
    """Retorna o id do status pelo nome (case insensitive, espaços e underscores equivalentes)."""
    key = name.strip().lower().replace("_", " ")
    status = _EPIC_STATUS_NAME_MAP.get(key)
    if status:
        return status.value
    raise ValueError(
        f"Status '{name}' não encontrado. Válidos: {', '.join(_EPIC_STATUS_NAME_MAP.keys())}"
    )




import os
from pathlib import Path

CONFIG_DIR = os.path.expanduser("~/.config/taiga-cli")
TOKEN_PATH = os.path.join(CONFIG_DIR, "token")
CONFIG_ENV_PATH = os.path.join(CONFIG_DIR, "config.env")
REFRESH_TOKEN_PATH = os.path.join(CONFIG_DIR, "refresh_token")


def save_refresh_token(refresh_token: str):
    os.makedirs(CONFIG_DIR, exist_ok=True)
    with open(REFRESH_TOKEN_PATH, "w") as f:
        f.write(refresh_token.strip())


def load_refresh_token() -> str | None:
    if os.path.exists(REFRESH_TOKEN_PATH):
        with open(REFRESH_TOKEN_PATH) as f:
            return f.read().strip()
    return None



def _read_config_env() -> dict:
    """Lê o arquivo config.env e retorna um dicionário com as variáveis."""
    config = {}
    if os.path.exists(CONFIG_ENV_PATH):
        with open(CONFIG_ENV_PATH) as f:
            for line in f:
                if line.strip() and not line.strip().startswith("#"):
                    if "=" in line:
                        k, v = line.strip().split("=", 1)
                        config[k.strip()] = v.strip()
    return config


def save_config_vars(api_url: str, project_id: str):
    os.makedirs(CONFIG_DIR, exist_ok=True)
    with open(CONFIG_ENV_PATH, "w") as f:
        f.write(f"TAIGA_API_URL={api_url}\n")
        f.write(f"TAIGA_PROJECT_ID={project_id}\n")


def save_token(token: str):
    os.makedirs(CONFIG_DIR, exist_ok=True)
    with open(TOKEN_PATH, "w") as f:
        f.write(token.strip())


def load_token() -> str | None:
    if os.path.exists(TOKEN_PATH):
        with open(TOKEN_PATH) as f:
            return f.read().strip()
    return None



def get_taiga_api_url() -> str:
    config = _read_config_env()
    return config.get("TAIGA_API_URL", "http://taiga-kanban/api/v1")



def get_taiga_project_id() -> str | None:
    config = _read_config_env()
    return config.get("TAIGA_PROJECT_ID")



from dataclasses import dataclass, field
from typing import List, Optional


@dataclass
class Task:
    id: str
    subject: str
    status: str
    story_id: Optional[str] = None
    tags: List[str] = field(default_factory=list)


@dataclass
class Story:
    id: str
    subject: str
    status: str
    epic_id: Optional[str] = None
    tags: List[str] = field(default_factory=list)


@dataclass
class Epic:
    id: str
    subject: str
    status: str
    initiative: Optional[str] = None
    tags: List[str] = field(default_factory=list)


@dataclass
class Initiative:
    id: str
    name: str
    order: int



from typing import List

from .api import TaigaAPI
from .models import Epic, Initiative, Story, Task


class TaigaService:
    def get_story_by_tag_full(self, tag: str, project_id: int) -> dict:
        """
        Busca o dict completo do primeiro user story do projeto que contenha a tag (case insensitive) na lista de tags.
        Retorna o dict do story (com id, version, etc).
        """
        tag_norm = tag.lower()
        data = self.api.get("userstories", {"project": project_id})
        for s in data:
            tags = [t[0].lower() for t in s.get("tags", []) if t and t[0]]
            if tag_norm in tags:
                return s
        raise ValueError(f"Nenhum user story com a tag '{tag}' encontrado no projeto {project_id}.")

    def get_epic_by_tag_full(self, tag: str, project_id: int) -> dict:
        """
        Busca o dict completo do primeiro épico do projeto que contenha a tag (case insensitive) na lista de tags.
        Retorna o dict do épico (com id, version, etc).
        """
        tag_norm = tag.lower()
        data = self.api.get("epics", {"project": project_id})
        for e in data:
            tags = [t[0].lower() for t in e.get("tags", []) if t and t[0]]
            if tag_norm in tags:
                return e
        raise ValueError(f"Nenhum épico com a tag '{tag}' encontrado no projeto {project_id}.")

    def get_epic_id_by_tag(self, tag: str, project_id: int) -> str:
        """
        Busca o id do primeiro épico do projeto que contenha a tag (case insensitive) na lista de tags.
        """
        tag_norm = tag.lower()
        epics = self.list_epics(project_id)
        for epic in epics:
            tags = getattr(epic, "tags", [])
            if tag_norm in tags:
                return epic.id
        raise ValueError(f"Nenhum épico com a tag '{tag}' encontrado no projeto {project_id}.")

    def __init__(self, api: TaigaAPI):
        self.api = api

    def list_tasks(self, project_id: int) -> List[Task]:
        data = self.api.get("tasks", {"project": project_id})
        return [Task(id=str(t["id"]), subject=t["subject"], status=t["status"]) for t in data]

    def list_stories(self, project_id: int) -> List[Story]:
        data = self.api.get("userstories", {"project": project_id})
        return [Story(id=str(s["id"]), subject=s["subject"], status=s["status"]) for s in data]

    def list_epics(self, project_id: int) -> List[Epic]:
        data = self.api.get("epics", {"project": project_id})
        return [
            Epic(
                id=str(e["id"]),
                subject=e["subject"],
                status=e["status"],
                tags=[t[0] for t in e.get("tags", [])],
            )
            for e in data
        ]

    def list_initiatives(self, project_id: int) -> List[Initiative]:
        data = self.api.get(f"projects/{project_id}")["swimlanes"]
        return [Initiative(id=str(i["id"]), name=i["name"], order=i["order"]) for i in data]
